//~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~

use crate::{s5, simu};

pub struct Turn {}

impl Turn {
    pub fn new() -> Self {
        Self {}
    }
}

impl simu::Interactor for Turn {
    fn click(
        &self,
        entity: &mut simu::Entity,
        _mouse: &s5::Point,
    ) {
        let kinetics = entity.kinetics_mut();
        let x_speed = kinetics.x_speed;
        let y_speed = kinetics.y_speed;
        kinetics.x_speed = -y_speed;
        kinetics.y_speed = x_speed;
        kinetics.angular_speed *= -1.0;
    }
}

//~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~

pub struct FastForward {
    dt: f64,
}

impl FastForward {
    pub fn new(dt: f64) -> Self {
        Self { dt }
    }
}

impl simu::Interactor for FastForward {
    fn click(
        &self,
        entity: &mut simu::Entity,
        _mouse: &s5::Point,
    ) {
        let kinetics = entity.kinetics_mut();
        kinetics.integrate(self.dt);
    }
}

//~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~

pub struct Push {
    speed: f64,
}

impl Push {
    pub fn new(speed: f64) -> Self {
        Self { speed }
    }
}

impl simu::Interactor for Push {
    fn click(
        &self,
        entity: &mut simu::Entity,
        mouse: &s5::Point,
    ) {
        let factor = self.speed / entity.bounding_radius();
        let kinetics = entity.kinetics_mut();
        let dx = kinetics.x - mouse.x;
        let dy = kinetics.y - mouse.y;
        kinetics.x_speed = factor * dx;
        kinetics.y_speed = factor * dy;
    }
}

//~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~

pub struct Chromo {}

impl Chromo {
    pub fn new() -> Self {
        Self {}
    }
}

impl simu::Interactor for Chromo {
    fn click(
        &self,
        entity: &mut simu::Entity,
        _mouse: &s5::Point,
    ) {
        let color = entity.color_mut();
        *color = s5::Color::new(color.g, color.b, color.r);
    }
}

//~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~

pub struct Pair {
    first: Box<dyn simu::Interactor>,
    second: Box<dyn simu::Interactor>,
}

impl Pair {
    pub fn new(
        first: Box<dyn simu::Interactor>,
        second: Box<dyn simu::Interactor>,
    ) -> Self {
        Self { first, second }
    }
}

impl simu::Interactor for Pair {
    fn click(
        &self,
        entity: &mut simu::Entity,
        mouse: &s5::Point,
    ) {
        self.first.click(entity, mouse);
        self.second.click(entity, mouse);
    }
}

//~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
