//~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~

use crate::{s5, simu};

pub struct Rectangle {
    size: s5::Rect,
    filled: bool,
}

impl Rectangle {
    pub fn new(
        size: s5::Rect,
        filled: bool,
    ) -> Self {
        Self { size, filled }
    }
}

impl simu::Shape for Rectangle {
    fn bounding_radius(&self) -> f64 {
        0.5 * (self.size.w * self.size.w + self.size.h * self.size.h).sqrt()
    }

    fn draw(
        &self,
        entity: &simu::Entity,
        screen: &mut s5::Screen,
    ) {
        let kinetics = entity.kinetics();
        let point = s5::Point {
            x: kinetics.x,
            y: kinetics.y,
        };
        let inner_size = s5::Rect {
            w: self.size.w * 0.75,
            h: self.size.h * 0.75,
        };
        screen.rectangle(
            &point,
            &self.size,
            if self.filled { None } else { Some(&inner_size) },
            kinetics.angle,
            entity.color(),
        );
    }
}

//~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~

pub struct Circle {
    radius: f64,
    filled: bool,
}

impl Circle {
    pub fn new(
        radius: f64,
        filled: bool,
    ) -> Self {
        Self { radius, filled }
    }
}

impl simu::Shape for Circle {
    fn bounding_radius(&self) -> f64 {
        self.radius
    }

    fn draw(
        &self,
        entity: &simu::Entity,
        screen: &mut s5::Screen,
    ) {
        let kinetics = entity.kinetics();
        let point = s5::Point {
            x: kinetics.x,
            y: kinetics.y,
        };
        let size = s5::Rect {
            w: self.radius * 2.0,
            h: self.radius * 2.0,
        };
        let inner_size = s5::Rect {
            w: size.w * 0.75,
            h: size.h * 0.75,
        };
        screen.ellipse(
            &point,
            &size,
            if self.filled { None } else { Some(&inner_size) },
            kinetics.angle,
            entity.color(),
        );
    }
}

//~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
